@@ -130,7 +130,7 @@ from django.core.exceptions import ValidationError
 # noinspection PyUnresolvedReferences
 from django.db.models import TextField
 
-from cardinal_pythonlib.json.serialize import json_decode, json_encode
+from cardinal_pythonlib.json_utils.serialize import json_decode, json_encode
 
 
 # =============================================================================
